/*
******************************************************************************
File:		main.cpp
Info:		     
			1-Wire    UART.
			  MCU STM32F100RB.
			    1-Wire   
			PA2 (USART2_TX)  PA3 (USART2_RX)     
			4.7     .

			  ,    
			 DS18B20 (  , ..   
			     ,  
			   ).
******************************************************************************
*/

/**
*  IMPORTANT NOTE!
*  The symbol VECT_TAB_SRAM needs to be defined when building the project
*  if code has been located to RAM and interrupts are used.
*  Otherwise the interrupt table located in flash will be used.
*  See also the <system_*.c> file and how the SystemInit() function updates
*  SCB->VTOR register.
*  E.g.  SCB->VTOR = 0x20000000;
*/

#include "stm32f10x.h"

// ,   UART     1-Wire.
#define OW_RESET			0xF0
#define OW_W0				0x00
#define OW_W1				0xFF
#define OW_R				0xFF

//  UART    1-Wire    .
#define OW_RESET_SPEED		9600
#define OW_TRANSFER_SPEED	115200

// ,   ow_reset()
#define OW_NO_DEVICE		0
#define OW_OK				1

//     .
void init_sys_tick()
{
	//    .
	SysTick->LOAD=0x00FFFFFF;
	//  .
	SysTick->CTRL=SysTick_CTRL_CLKSOURCE|SysTick_CTRL_ENABLE;
}

//      us .
void udelay(int us)
{
	//       .
	int t0=SysTick->VAL;

	//        .
	int64_t n=SystemCoreClock;
	n*=us;
	n/=1000000;

	//     .
	while(n>0)
	{
		int t=SysTick->VAL;
		int r=t0-t;
		if(r<=0)
			r+=SysTick->LOAD+1;
		n-=r;
		t0=t;
	}
}

//    USART2   speed (/).
//   ,   / .
void usart2_change_speed(uint32_t speed)
{
	USART2->BRR=SystemCoreClock/speed;
}

//    ch  USART2;
//          
//   (   ,    ).
void usart2_send_byte(uint8_t ch) {
	while(!(USART2->SR&USART_SR_TXE)) {}
		USART2 -> DR = ch;
}

//   ,  USART2;
//     USART2 ,    .
char usart2_receive_byte()
{
	while(!(USART2->SR&USART_SR_RXNE)) {}
	return USART2->DR;
}

//   USART2   USART2 
// (PA2 - USART2_TX; PA3 - USART2_RX).
void usart2_init()
{
	//   USART2  GPIOA;  
	//  GPIOA:
	// PA2 (USART2_TX) -     ;
	// PA3 (USART2_RX) - .
	RCC->APB1ENR|=RCC_APB1ENR_USART2EN;
	RCC->APB2ENR|=RCC_APB2ENR_IOPAEN;
	uint32_t tmp=GPIOA->CRL;
	tmp&=~0xFF00;
	// TX, PA2: out 10MHz, alt. open drain, 1101=0xD
	// RX, PA3: in, floating: 0100=0x4
	tmp|=0x4D00;
	GPIOA->CRL=tmp;

	//     USART,   
	//   1-Wire .
	usart2_change_speed(OW_TRANSFER_SPEED);

	//  USART;    .
	// ,       1-Wire 
	//     1.
	USART2->CR1|=USART_CR1_UE;
	USART2->CR1|=USART_CR1_TE|USART_CR1_RE;

	//          ,
	//     .
	USART2->DR;
}

// ***************************************************************************
//      1-Wire.
// ***************************************************************************

//       1-Wire;
//  OW_OK,       ;
//  OW_NO_DEVICE,      .
int ow_reset()
{
	//      
	// USART   9600.
	usart2_change_speed(OW_RESET_SPEED);
	//      USART  0xF0.
	usart2_send_byte(OW_RESET);
	//    :    
	//   -   ;
	//    -   .
	char r=usart2_receive_byte();
	//   USART    .
	usart2_change_speed(OW_TRANSFER_SPEED);

	return (r==OW_RESET)?OW_NO_DEVICE:OW_OK;
}

//      1-Wire (  
//  b).
//      ,     1.
uint8_t ow_send_bit(uint8_t b)
{
	usart2_send_byte((b&1)?OW_W1:OW_W0);
	return (usart2_receive_byte()==OW_W1)?1:0;
}

//    b   1-Wire.
//      0xFF.
uint8_t ow_send(uint8_t b)
{
	int r=0;
	for(int i=0; i<8; i++, b>>=1)
	{
		r>>=1;
		if(ow_send_bit(b))
			r|=0x80;
	}
	return r;
}

//    buf  size    1-Wire
void ow_send(const void *buf, unsigned int size)
{
	const uint8_t *p=(const uint8_t *)buf;
	for(unsigned int i=0; i<size; i++, p++)
		ow_send(*p);
}

//      1-Wire;
//  ow_send(0xFF)
inline uint8_t ow_receive()
{
	return ow_send(0xFF);
}

//  size          buf.
void ow_receive(void *buf, unsigned int size)
{
	uint8_t *p=(uint8_t *)buf;
	for(unsigned int i=0; i<size; i++, p++)
		*p=ow_send(0xFF);
}

//         
//       DS18B20;
// ,   -     1-Wire;
//    .
int main(void)
{
	//   .
	init_sys_tick();

	//    DS18B20,   ,   
	//       .
	udelay(1000);	//  1.

	//  USART2,    1-Wire 
	//   PA2 (TX), PA3 (RX),  USART2.
	usart2_init();

	//     .
	while(true)
	{
		// /  .
		if(ow_reset()!=OW_OK)
			break;
		//     :
		// SKIP ROM (0xCC), CONVERT T (0x44).
		ow_send("\xCC\x44", 2);

		//     ADC;
		//       , 
		//         
		//  0,     1.
		while(ow_send_bit(1)==0) {udelay(50000);}

		//     
		// SKIP ROM (0xCC), READ SCRATCHPAD (0xBE)
		//    .
		ow_reset();
		ow_send("\xCC\xBE", 2);

		//   ,     -  
		//   ,   16.
		//       SCRATCHPAD 
		//  CRC.
		int16_t tx16=0;
		ow_receive(&tx16, 2);

		//   ,     
		//  .
		// float t=tx16/16.0;
		float t=tx16; t/=16;
	}

	/* Infinite loop */
	int i = 0;
	while (1)
	{
		i++;
	}
}

